@@ -7,7 +7,7 @@ use warnings 'all';
 ###############################################################################
 # METADATA
 our $AUTHORITY = 'cpan:DOUGDUDE';
-our $VERSION   = '0.006';
+our $VERSION   = '0.007';
 
 ###############################################################################
 # MOOSE
@@ -20,8 +20,8 @@ with 'MooseX::Clone';
 
 ###############################################################################
 # MODULES
-use Convert::Binary::C 0.74;
-use Digest::CRC qw(crc32);
+use Digest::CRC ();
+use Net::NSCA::Client::ServerConfig ();
 use Readonly 1.03;
 
 ###############################################################################
@@ -36,8 +36,8 @@ Readonly our $MAX_SERVICE_MESSAGE_LENGTH     => 512;
 
 ###############################################################################
 # OVERLOADED FUNCTIONS
-__PACKAGE__->meta->add_package_symbol(q{&()}  => sub {                  });
-__PACKAGE__->meta->add_package_symbol(q{&(""} => sub { shift->to_string });
+__PACKAGE__->meta->add_package_symbol(q{&()}  => sub {                   });
+__PACKAGE__->meta->add_package_symbol(q{&(""} => sub { shift->raw_packet });
 
 ###############################################################################
 # PRIVATE CONSTANTS
@@ -59,6 +59,20 @@ has packet_version => (
 	default       => 3,
 	documentation => q{The version of the packet being transmitted},
 );
+has raw_packet => (
+	is  => 'ro',
+	isa => 'Str',
+
+	lazy    => 1,
+	builder => '_build_raw_packet',
+);
+has server_config => (
+	is  => 'ro',
+	isa => 'Net::NSCA::Client::ServerConfig',
+
+	default => sub { Net::NSCA::Client::ServerConfig->new },
+	traits  => [qw(Clone)],
+);
 has service_description => (
 	is  => 'ro',
 	isa => 'Str',
@@ -92,16 +106,32 @@ around BUILDARGS => sub {
 	if (@args == 1 && !ref $args[0]) {
 		# This should be the packet as a string, so get the new
 		# args from this string
-		@args = _constructor_options_from_string($args[0]);
+		@args = (raw_packet => $args[0]);
 	}
 
-	# Call the original method
-	return $class->$original_method(@args);
+	# Call the original method to get args HASHREF
+	my $args = $class->$original_method(@args);
+
+	if (exists $args->{raw_packet}) {
+		# The packet was provided to the constructor
+		$args = {
+			%{$args}, # Given arguments
+			_constructor_options_from_string($args->{raw_packet}, $args->{server_config})
+		};
+	}
+
+	return $args;
 };
 
 ###############################################################################
 # METHODS
 sub to_string {
+	return shift->raw_packet;
+}
+
+###############################################################################
+# PRIVATE METHODS
+sub _build_raw_packet {
 	my ($self) = @_;
 
 	# Create a HASH of the value to be provided to the pack
@@ -115,18 +145,15 @@ sub to_string {
 		svc_description => $self->service_description,
 	);
 
-	# Get the packer data object
-	my $packer = _data_packet_struct();
-
 	# To construct the packet, we will use the pack method from the
 	# Convert::Binary::C object
-	my $packet = $packer->pack(data_packet_struct => \%pack_options);
-
-	# Calculate the CRC32 value for the packet
-	$pack_options{crc32_value} = crc32($packet);
+	my $packet = $self->server_config->pack_data_packet(\%pack_options);
 
 	# Repack the packet with the CRC32 value
-	$packet = $packer->pack(data_packet_struct => \%pack_options);
+	$self->server_config->repack_data_packet(\$packet, {
+		# Calculate the CRC32 value for the packet
+		crc32_value => Digest::CRC::crc32($packet),
+	});
 
 	# Return the packet
 	return $packet;
@@ -135,13 +162,24 @@ sub to_string {
 ###############################################################################
 # PRIVATE FUNCTIONS
 sub _constructor_options_from_string {
-	my ($packet) = @_;
+	my ($packet, $server_config) = @_;
 
-	# Get the packer data object
-	my $packer = _data_packet_struct();
+	if (!defined $server_config) {
+		# Get the attribute from the class
+		my $attr = __PACKAGE__->meta->find_attribute_by_name('server_config');
+
+		# Set to the class default
+		$server_config = $attr->is_default_a_coderef ? $attr->default->()
+		                                             : $attr->default
+		                                             ;
+	}
+
+	if (!_is_packet_valid($packet, $server_config)) {
+		Moose->throw_error('Provided packet is not valid');
+	}
 
 	# Unpack the data packet
-	my $unpacket = $packer->unpack(data_packet_struct => $packet);
+	my $unpacket = $server_config->unpack_data_packet($packet);
 
 	# Return the options for the constructor
 	return (
@@ -153,110 +191,20 @@ sub _constructor_options_from_string {
 		unix_timestamp      => $unpacket->{timestamp      },
 	);
 }
-sub _data_packet_struct {
-	# Create a C object
-	my $c = _setup_c_object();
+sub _is_packet_valid {
+	my ($packet, $server_config) = @_;
 
-	# Add the data_packet_struct structure
-	$c->parse(<<"ENDC");
-		struct data_packet_struct {
-			int16_t   packet_version;
-			u_int32_t crc32_value;
-			u_int32_t timestamp;
-			int16_t   return_code;
-			char      host_name[$MAX_HOSTNAME_LENGTH];
-			char      svc_description[$MAX_SERVICE_DESCRIPTION_LENGTH];
-			char      plugin_output[$MAX_SERVICE_MESSAGE_LENGTH];
-		};
-ENDC
+	# Extract the CRC from the packet
+	my $crc32 = $server_config->unpack_data_packet($packet)->{crc32_value};
 
-	# Add the string hooks to all the string members
-	foreach my $string_member (qw(host_name svc_description plugin_output)) {
-		# XXX: THE RANDOMNESS OF THE STRING BREAKS CRC32
-		# XXX: $c->tag("data_packet_struct.$string_member", Hooks => {
-		# XXX: 	pack   => [\&_string_randpad_pack, $c->arg(qw(DATA SELF TYPE)), 'data_packet_struct'],
-		# XXX: 	unpack =>  \&_string_unpack,
-		# XXX: });
-		$c->tag("data_packet_struct.$string_member", Format => 'String');
-	}
+	# Repack the packet with CRC32 as zero so that the CRC32 can
+	# be recalculated
+	$server_config->repack_data_packet(\$packet, {
+		crc32_value => 0,
+	});
 
-	return $c;
-}
-sub _setup_c_object {
-	my ($c) = @_;
-
-	# If no object provided, create a new one
-	$c ||= Convert::Binary::C->new;
-
-	# Set the memory structure to store in network order
-	$c->ByteOrder('BigEndian');
-
-	# The alignment always seems to be 4 bytes, so set the alignment here
-	$c->Alignment($BYTES_FOR_32BITS);
-
-	# Create a HASH of sizes to types
-	my %int_sizes;
-
-	$int_sizes{$c->sizeof('int'          )} = 'int';
-	$int_sizes{$c->sizeof('long int'     )} = 'long int';
-	$int_sizes{$c->sizeof('long long int')} = 'long long int';
-	$int_sizes{$c->sizeof('short int'    )} = 'short int';
-
-	# Check the needed types are present
-	if (!exists $int_sizes{$BYTES_FOR_16BITS}) {
-		confess 'Your platform does not have any C data type that is 16 bits';
-	}
-	if (!exists $int_sizes{$BYTES_FOR_32BITS}) {
-		confess 'Your platform does not have any C data type that is 32 bits';
-	}
-
-	# Now that the sizes are known, set up various typedefs
-	$c->parse(sprintf 'typedef %s int16_t;'           , $int_sizes{$BYTES_FOR_16BITS});
-	$c->parse(sprintf 'typedef unsigned %s u_int16_t;', $int_sizes{$BYTES_FOR_16BITS});
-	$c->parse(sprintf 'typedef %s int32_t;'           , $int_sizes{$BYTES_FOR_32BITS});
-	$c->parse(sprintf 'typedef unsigned %s u_int32_t;', $int_sizes{$BYTES_FOR_32BITS});
-
-	# Return the object
-	return $c;
-}
-sub _string_randpad_pack {
-	my ($string, $c, $type, $struct) = @_;
-
-	if (defined $struct) {
-		$type = sprintf '%s.%s', $struct, $type;
-	}
-
-	# Cut off the NULL and anything after it
-	($string) = $string =~ m{\A ([^\0]+)}msx;
-
-	# Add NULL to the end of the string
-	$string .= chr 0;
-
-	# Get the max length
-	my $max_length = $c->sizeof($type);
-
-	# Check if the string is too long
-	if ($max_length < length $string) {
-		confess sprintf 'The string provided to %s is too long. Max length is %s bytes',
-			$type, $max_length - 1;
-	}
-
-	# Create an array of letters and numbers
-	my @letters_and_numbers = ('a'..'z', 'A'..'Z', '0'..'9');
-
-	# Pad the remaining space with random ASCII characters
-	while ($max_length > length $string) {
-		$string .= $letters_and_numbers[int rand @letters_and_numbers];
-	}
-
-	# Return the string
-	return [unpack 'c*', $string];
-}
-sub _string_unpack {
-	my ($c_string_struct) = @_;
-
-	# Return the Perl string
-	return pack 'Z*', @{$c_string_struct->{buf}};
+	# Packet is valid if the CRC32 values are the same
+	return $crc32 == Digest::CRC::crc32($packet);
 }
 
 ###############################################################################
@@ -273,7 +221,7 @@ Net::NSCA::Client::DataPacket - Implements data packet for the NSCA protocol
 
 =head1 VERSION
 
-This documentation refers to L<Net::NSCA::Client::DataPacket> version 0.006
+This documentation refers to version 0.007
 
 =head1 SYNOPSIS
 
@@ -290,7 +238,9 @@ This documentation refers to L<Net::NSCA::Client::DataPacket> version 0.006
   );
 
   # Create a packet recieved from over the network
-  my $recieved_packet = Net::NSCA::Client::DataPacket->new($recieved_data);
+  my $recieved_packet = Net::NSCA::Client::DataPacket->new(
+      raw_packet => $recieved_data,
+  );
 
 =head1 DESCRIPTION
 
@@ -344,6 +294,12 @@ This is the version of the packet to be sent. A few different NSCA servers use
 slightly different version numbers, but the rest of the packet is the same.
 If not specified, this will default to 3.
 
+=head2 raw_packet
+
+This is the raw packet to send over the network. Providing this packet to
+the constructor will automatically populate all other attributes and so
+they are B<not> required if this attribute is provided.
+
 =head2 service_description
 
 B<Required>
@@ -363,7 +319,14 @@ B<Required>
 
 This is the status of the service that will be given to Nagios. It is
 recommended to use one of the C<$STATUS_> constants provided by
-L<Net::NSCA::Client>.
+L<Net::NSCA::Client|Net::NSCA::Client>.
+
+=head2 server_config
+
+This specifies the configuration of the remote NSCA server. See
+L<Net::NSCA::Client::ServerConfig|Net::NSCA::Client::ServerConfig> for details
+about using this. Typically this does not need to be specified unless the
+NSCA server was compiled with customizations.
 
 =head2 unix_timestamp
 
@@ -382,19 +345,19 @@ representation is what will be sent over the network.
 
 =over
 
-=item * L<Convert::Binary::C> 0.74
+=item * L<Digest::CRC|Digest::CRC>
 
-=item * L<Digest::CRC>
+=item * L<Moose|Moose> 0.89
 
-=item * L<Moose> 0.89
+=item * L<MooseX::Clone|MooseX::Clone>
 
-=item * L<MooseX::Clone>
+=item * L<MooseX::StrictConstructor|MooseX::StrictConstructor> 0.08
 
-=item * L<MooseX::StrictConstructor> 0.08
+=item * L<Net::NSCA::Client::ServerConfig|Net::NSCA::Client::ServerConfig>
 
-=item * L<Readonly> 1.03
+=item * L<Readonly|Readonly> 1.03
 
-=item * L<namespace::clean> 0.04
+=item * L<namespace::clean|namespace::clean> 0.04
 
 =back
 
